'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _spacer = require('./spacer');

var _spacer2 = _interopRequireDefault(_spacer);

var _formatTreeNode = require('./formatTreeNode');

var _formatTreeNode2 = _interopRequireDefault(_formatTreeNode);

var _formatProp2 = require('./formatProp');

var _formatProp3 = _interopRequireDefault(_formatProp2);

var _propNameSorter = require('./propNameSorter');

var _propNameSorter2 = _interopRequireDefault(_propNameSorter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var recurse = function recurse(lvl, inline, options) {
  return function (element) {
    return (0, _formatTreeNode2.default)(element, inline, lvl, options);
  };
};

// FIXME: naming
var onlyOriginalProps = function onlyOriginalProps(leftProps, rightProps) {
  return function (propName) {
    var isIncludedInLeft = Object.keys(leftProps).includes(propName);
    return !isIncludedInLeft || isIncludedInLeft && leftProps[propName] !== rightProps[propName];
  };
};

var isInlineAttributeTooLong = function isInlineAttributeTooLong(attributes, inlineAttributeString, lvl, tabStop, maxInlineAttributesLineLength) {
  if (!maxInlineAttributesLineLength) {
    return attributes.length > 1;
  }

  return (0, _spacer2.default)(lvl, tabStop).length + inlineAttributeString.length > maxInlineAttributesLineLength;
};

var shouldRenderMultilineAttr = function shouldRenderMultilineAttr(attributes, inlineAttributeString, containsMultilineAttr, inline, lvl, tabStop, maxInlineAttributesLineLength) {
  return (isInlineAttributeTooLong(attributes, inlineAttributeString, lvl, tabStop, maxInlineAttributesLineLength) || containsMultilineAttr) && !inline;
};

// FIXME: Make this code more robust?
var mergePlainStringChildren = function mergePlainStringChildren(prev, currentNode) {
  var lastNode = prev[prev.length - 1];

  if (currentNode.type === 'number') {
    currentNode.type = 'string';
    currentNode.value = String(currentNode.value);
  }

  if (lastNode && lastNode.type === 'string' && currentNode.type === 'string') {
    prev[prev.length - 1].value += currentNode.value;
  } else {
    prev.push(currentNode);
  }

  return prev;
};

exports.default = function (node, inline, lvl, options) {
  var displayName = node.displayName,
      childrens = node.childrens,
      props = node.props,
      defaultProps = node.defaultProps;

  // FIXME: should be necessary now

  if (!displayName) throw new Error('Oups: displayName attribute is missing.');
  if (!childrens) throw new Error('Oups: childrens attribute is missing.');
  if (!props) throw new Error('Oups: props attribute is missing.');
  if (!defaultProps) throw new Error('Oups: defaultProps attribute is missing.');

  var filterProps = options.filterProps,
      maxInlineAttributesLineLength = options.maxInlineAttributesLineLength,
      showDefaultProps = options.showDefaultProps,
      sortProps = options.sortProps,
      tabStop = options.tabStop;


  var out = '<' + displayName;

  var outInlineAttr = out;
  var outMultilineAttr = out;
  var containsMultilineAttr = false;

  var visibleAttributeNames = [];

  Object.keys(props).filter(function (propName) {
    return filterProps.indexOf(propName) === -1;
  }).filter(onlyOriginalProps(defaultProps, props)).forEach(function (propName) {
    return visibleAttributeNames.push(propName);
  });

  Object.keys(defaultProps).filter(function (defaultPropName) {
    return filterProps.indexOf(defaultPropName) === -1;
  }).filter(function () {
    return showDefaultProps;
  }).filter(function (defaultPropName) {
    return !visibleAttributeNames.includes(defaultPropName);
  }).forEach(function (defaultPropName) {
    return visibleAttributeNames.push(defaultPropName);
  });

  var attributes = visibleAttributeNames.sort((0, _propNameSorter2.default)(sortProps));

  attributes.forEach(function (attributeName) {
    var _formatProp = (0, _formatProp3.default)(attributeName, Object.keys(props).includes(attributeName), props[attributeName], Object.keys(defaultProps).includes(attributeName), defaultProps[attributeName], inline, lvl, options),
        attributeFormattedInline = _formatProp.attributeFormattedInline,
        attributeFormattedMultiline = _formatProp.attributeFormattedMultiline,
        isMultilineAttribute = _formatProp.isMultilineAttribute;

    if (isMultilineAttribute) {
      containsMultilineAttr = true;
    }

    outInlineAttr += attributeFormattedInline;
    outMultilineAttr += attributeFormattedMultiline;
  });

  outMultilineAttr += '\n' + (0, _spacer2.default)(lvl, tabStop);

  if (shouldRenderMultilineAttr(attributes, outInlineAttr, containsMultilineAttr, inline, lvl, tabStop, maxInlineAttributesLineLength)) {
    out = outMultilineAttr;
  } else {
    out = outInlineAttr;
  }

  if (childrens && childrens.length > 0) {
    out += '>';
    lvl++;
    if (!inline) {
      out += '\n';
      out += (0, _spacer2.default)(lvl, tabStop);
    }

    var onlyStringChildren = childrens.reduce(function (prev, children) {
      if (!prev) {
        return false;
      }

      if (typeof children !== 'string' || typeof children !== 'number') {
        return true;
      }

      return false;
    }, true);

    out += childrens.reduce(mergePlainStringChildren, []).map(recurse(lvl, inline, options)).join('\n' + (0, _spacer2.default)(lvl, tabStop));

    if (!inline) {
      out += '\n';
      out += (0, _spacer2.default)(lvl - 1, tabStop);
    }
    out += '</' + displayName + '>';
  } else {
    if (!isInlineAttributeTooLong(attributes, outInlineAttr, lvl, tabStop, maxInlineAttributesLineLength)) {
      out += ' ';
    }

    out += '/>';
  }

  return out;
};
//# sourceMappingURL=formatReactElementNode.js.map